//
//  TrackingTests.swift
//  Research
//

import XCTest
@testable import Research
@testable import Research_UnitTest


@available(*,deprecated, message: "Will be deleted in a future version.")
class TrackingTests: XCTestCase {
    
    override func setUp() {
        super.setUp()
        // Put setup code here. This method is called before the invocation of each test method in the class.
        setupPlatformContext()
    }
    
    override func tearDown() {
        // Put teardown code here. This method is called after the invocation of each test method in the class.
        super.tearDown()
    }
    
    // WeeklyScheduleObject
    
    /// This test takes 13 seconds to run and it's just a sanity check. I added it to check that for any
    /// combination of locale and calendar, that the components for time and day of the week do not change.
    /// Leaving the disabled test in place in order to answer the question "But what about different
    /// calendars and locals?" syoung 02/22/2018
    func disable_testWeeklyScheduleItem_CalendarAssumptions() {
        // Check the assumption that current Calendar will not influence the day of the week or time of day.
        let now = Date()
        let weekday = Calendar.iso8601.component(.weekday, from: now)
        let hour = Calendar.iso8601.component(.hour, from: now)
        let minute = Calendar.iso8601.component(.minute, from: now)
        for identifier in Calendar.Identifier.all {
            var calendar = Calendar(identifier: identifier)
            for localeCode in Locale.availableIdentifiers {
                calendar.locale = Locale(identifier: localeCode)
                XCTAssertEqual(calendar.component(.weekday, from: now), weekday, "\(calendar)")
                XCTAssertEqual(calendar.component(.hour, from: now), hour, "\(calendar)")
                XCTAssertEqual(calendar.component(.minute, from: now), minute, "\(calendar)")
                XCTAssertEqual(calendar.weekdaySymbols.count, 7, "\(calendar)")
            }
        }
    }
    
    func testWeeklyScheduleItem_WeekdayOrdinal() {
        NSLocale.setCurrentTest(Locale(identifier: "en_US"))

        // Check assumptions for this Locale
        XCTAssertEqual(Locale.current.calendar.firstWeekday, 1)
        
        let weekdays = RSDWeekday.all.sorted()
        let weekdayOrder = weekdays.map { $0.rawValue }
        XCTAssertEqual(weekdayOrder, Array(1...7))
        
        let names = weekdays.compactMap { $0.text }
        XCTAssertEqual(names, ["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"])
        
        let shortNames = weekdays.compactMap { $0.shortText }
        XCTAssertEqual(shortNames, ["Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"])
    }
    
    func testWeeklyScheduleItem_WeekdayOrdinal_France() {
        NSLocale.setCurrentTest(Locale(identifier: "fr_FR"))
        
        // Check assumptions for this Locale
        XCTAssertEqual(Locale.current.calendar.firstWeekday, 2)
        
        let weekdays = RSDWeekday.all.sorted()
        let weekdayOrder = weekdays.map { $0.rawValue }
        XCTAssertEqual(weekdayOrder, [2,3,4,5,6,7,1])
        
        let names = weekdays.compactMap { $0.text }
        XCTAssertEqual(names, ["Lundi", "Mardi", "Mercredi", "Jeudi", "Vendredi", "Samedi", "Dimanche"])
    }
    
    func testWeeklyScheduleItem_WeekdayOrdinal_Uzbec() {
        NSLocale.setCurrentTest(Locale(identifier: "uz_Arab"))
        
        // Check assumptions for this Locale
        XCTAssertEqual(Locale.current.calendar.firstWeekday, 7)
        
        let weekdays = RSDWeekday.all.sorted()
        let weekdayOrder = weekdays.map { $0.rawValue }
        XCTAssertEqual(weekdayOrder, [7,1,2,3,4,5,6])
    }
    
    func testWeeklyScheduleItem_TimeOfDay() {
        var item = RSDWeeklyScheduleObject()
        XCTAssertNil(item.timeOfDayString)
        
        var timeComponents = DateComponents()
        timeComponents.hour = 8
        timeComponents.minute = 0
        item.setTime(from: timeComponents)
        XCTAssertEqual(item.timeOfDayString, "08:00")
        XCTAssertEqual(item.timeComponents, timeComponents)
        
        item.setTime(from: Date())
        XCTAssertNotNil(item.timeOfDayString)
        XCTAssertNotNil(item.timeOfDay)
        XCTAssertNotNil(item.timeComponents)
    }
    
    func testWeeklyScheduleItem_NotificationTriggers() {
        var item = RSDWeeklyScheduleObject()
        item.timeOfDayString = "08:00"
        item.daysOfWeek = [.monday, .wednesday, .friday]
        
        let triggers = item.notificationTriggers()
        XCTAssertEqual(triggers.count, 3)
        for trigger in triggers {
            XCTAssertEqual(trigger.hour, 8, "\(trigger)")
            XCTAssertEqual(trigger.minute, 0, "\(trigger)")
            XCTAssertNil(trigger.year)
            XCTAssertNil(trigger.month)
            XCTAssertNil(trigger.day)
            XCTAssertNil(trigger.weekdayOrdinal)
        }
        let weekdays = triggers.rsd_flatMapSet { $0.weekday}
        XCTAssertEqual(weekdays, [2,4,6])
    }
    
    func testWeeklyScheduleFormatter_Daily() {
        NSLocale.setCurrentTest(Locale(identifier: "en_US"))
        
        var item1 = RSDWeeklyScheduleObject()
        item1.timeOfDayString = "08:00"
        
        var item2 = RSDWeeklyScheduleObject()
        item2.timeOfDayString = "20:30"
        
        let items = [item1, item2]
        let formatter = RSDWeeklyScheduleFormatter()
        
        formatter.style = .long
        XCTAssertEqual(formatter.string(from: items), "Every day at 8:00 AM and 8:30 PM")
        
        formatter.style = .medium
        XCTAssertEqual(formatter.string(from: items), "8:00 AM, 8:30 PM\nEvery day")
        
        formatter.style = .short
        XCTAssertEqual(formatter.string(from: items), "8:00 AM, 8:30 PM, Every day")
    }
    
    func testWeeklyScheduleFormatter_SameDays() {
        NSLocale.setCurrentTest(Locale(identifier: "en_US"))
        
        var item1 = RSDWeeklyScheduleObject()
        item1.timeOfDayString = "08:00"
        item1.daysOfWeek = [.monday, .wednesday, .friday]
        
        var item2 = RSDWeeklyScheduleObject()
        item2.timeOfDayString = "20:30"
        item2.daysOfWeek = [.monday, .wednesday, .friday]
        
        let items = [item1, item2]
        let formatter = RSDWeeklyScheduleFormatter()
        
        formatter.style = .long
        XCTAssertEqual(formatter.string(from: items), "Monday, Wednesday, and Friday at 8:00 AM and 8:30 PM")
        
        formatter.style = .medium
        XCTAssertEqual(formatter.string(from: items), "8:00 AM, 8:30 PM\nMonday, Wednesday, Friday")
        
        formatter.style = .short
        XCTAssertEqual(formatter.string(from: items), "8:00 AM, 8:30 PM, Mon, Wed, Fri")
    }
    
    func testWeeklyScheduleFormatter_DifferentDays() {
        NSLocale.setCurrentTest(Locale(identifier: "en_US"))
        
        var item1 = RSDWeeklyScheduleObject()
        item1.timeOfDayString = "08:00"
        
        var item2 = RSDWeeklyScheduleObject()
        item2.timeOfDayString = "20:30"
        item2.daysOfWeek = [.tuesday, .thursday]
        
        let items = [item1, item2]
        let formatter = RSDWeeklyScheduleFormatter()
        
        formatter.style = .long
        XCTAssertEqual(formatter.string(from: items), "Every day at 8:00 AM\nTuesday and Thursday at 8:30 PM")
        
        formatter.style = .medium
        XCTAssertEqual(formatter.string(from: items), "8:00 AM, Every day\n8:30 PM, Tue, Thu")
        
        formatter.style = .short
        XCTAssertEqual(formatter.string(from: items), "8:00 AM, Every day\n8:30 PM, Tue, Thu")
    }
}

extension Calendar.Identifier {

    public static var all: Set<Calendar.Identifier> {
        return [.gregorian, .buddhist, .chinese, .coptic, .ethiopicAmeteMihret, .ethiopicAmeteAlem, .hebrew, .iso8601, .indian, .islamic, .islamicCivil, .japanese, .persian, .republicOfChina, .islamicTabular, .islamicUmmAlQura]
    }
}

